@@ -27,9 +27,13 @@
        data
      * CutOffFile /' '/ 
        data
+     * GeomagFile /' '/
+       data
      * UserHookc/MAX_USERHOOKC*' '/
        data
      * UserHooki/MAX_USERHOOKI*99999999/
        data
      * UserHookr/MAX_USERHOOKR*1.d65/
+       data
+     * AtmosFile/' '/
       end
