@@ -1,5 +1,6 @@
 c         to be called when the end of execution status is reached.
       subroutine cendRun
+      implicit none
 c           save current status.      
           call csaveStatus 
 c            user hook
@@ -7,6 +8,7 @@ c            user hook
       end
 c       ***************************************** stave  cont info
       subroutine csaveStatus
+      implicit none
 #include "Zmanagerp.h"
       
       PrevEventNo = EventNo    ! current event no.
@@ -17,11 +19,29 @@ c       ***************************************** stave  cont info
       end
 c       ***************************************** print final status
       subroutine cprintStatus
+      implicit none
 #include "Zmanagerp.h"
+#include  "Zptcl.h"
+#include  "Zprimary.h"
+#include  "Zprimaryv.h"
 
 
       character*80 msg
+      integer i
+
       write(msg, *) ' No of cummulative events =', PrevEventNo, 
      *      ' No of events in this run=', EventsInTheRun
       call cerrorMsg(msg, 1)
+      write(msg, *) ' comp.    sampled    accepted'
+      call cerrorMsg(msg, 1)
+      do i = 1, Prim.no_of_comps
+         write(msg, '(i4,2i12)')
+     *   i, Prim.NoOfSampComp(i,1), Prim.NoOfSampComp(i,2)
+         call cerrorMsg(msg, 1)
+      enddo
       end
+
+
+
+
+
