@@ -11,7 +11,7 @@ c              eventLoop
 
       record /track/ incident
       integer jold
-      integer fin  
+      integer fin, jeof
 
       do while (EventNo .lt. abs( DestEventNo(1)) .and. 
      *      EventsInTheRun .lt. abs( DestEventNo(2)))
@@ -21,7 +21,8 @@ c             switch to the random number generator 1
          call rndsw(jold, 1)
          if(Job .eq. 'flesh') then
 c                get random seed at skelton making
-            call creadSeed(SeedSave, EventNo)
+            call creadSeed(SeedSave, EventNo, jeof)
+            if( jeof .ne. 0 ) goto 10
 c                reset the seed.
             call rnd1r(SeedSave)
          else
@@ -68,22 +69,25 @@ c        at skelton making time
       end
 c     ****************************************************
 c          read iformation for initial random number at flesh time
-      subroutine creadSeed(ir, no)
+      subroutine creadSeed(ir, no, jeof)
       implicit none
 #include "Zmanagerp.h"      
 
-      integer ir(2), no
+      integer ir(2), no, jeof
 
       integer nn
       integer count
       data count/0/
       save count
 
+      jeof = 0
       do while(.true.)
          read(SeedFileDev, *, end=100) ir, nn
          if(nn .ge. EventNo) goto 200  ! skip already processed ones
       enddo
+
  100  continue
+      jeof =1
       if(count .eq. 0) then
          count = count + 1
          call cerrorMsg(
