@@ -40,6 +40,7 @@ c            (in this case Za1ry == 'is' guaranteed).
 c           
          if(ObsPlane .eq. spherical) then
 c              reset position and angle.
+
             call cresetPosAng(incident)
          endif    
          if(CutOffFile .ne. ' ') then
@@ -82,6 +83,7 @@ c
 c
 #include  "Zglobalc.h"
 #include  "Ztrack.h"  
+#include  "Ztrackp.h"
 #include  "Zobs.h"  
 #include  "Zobsv.h"  
 #include  "Zincidentp.h"
@@ -91,6 +93,11 @@ c
       record /coord/ incipos, dir1, dir2
       logical first
       real*8 len, cs, sn, sint, u, oa1, oa2
+
+c     @@@@@@@@@@@@@For  bug correction to ObsPlane=3
+      real*8 cosx, ux
+c     @@@@@@@@@@@@@@
+
       data first/.true./
       save first
       
@@ -99,33 +106,50 @@ c         fix position
       oa2 = IMAG_P(Azimuth) 
 c        if we don't use oa1, oa2 but use real(..) directly in
 c        the subroutine call, Absoft compiler give always 0 for
-c        oa1  !!!!!!!!
+c        oa1  
 c
-      if(first) then
-         call cuonSphere(1, PolarInjPos.r(3), PolarInjPos.r(1),
-     *     PolarInjPos.r(2), oa1, oa2,  incident.pos.xyz)
-         first =.false.
-      else
-c              this is quicker a bit
-         call cuonSphere(2, PolarInjPos.r(3), PolarInjPos.r(1),
-     *   PolarInjPos.r(2),  oa1, oa2, incident.pos.xyz)
-      endif
-c      fix angle around zenith at incident.pos.xyz
-c         convert PolarInjPos to xyz vector
-      call ctransCoord2('xyz', incident.pos.xyz, incipos)
+      cosx = 2.      ! @@@@@@@@@
+      ux = 0.        ! @@@@@@@@@
+      do while (cosx .gt. -ux)  !@@@@@@@@@
+         if(first) then
+            call cuonSphere(1, PolarInjPos.r(3), PolarInjPos.r(1),
+     *           PolarInjPos.r(2), oa1, oa2,  incident.pos.xyz)
+            first =.false.
+         else
+c               this is quicker a bit
+            call cuonSphere(2, PolarInjPos.r(3), PolarInjPos.r(1),
+     *      PolarInjPos.r(2),  oa1, oa2, incident.pos.xyz)
+         endif
+c           fix angle around zenith at incident.pos.xyz
+c            convert PolarInjPos to xyz vector
+         call ctransCoord2('xyz', incident.pos.xyz, incipos)
 c            convert its direction to direction cos
-      call c3DV2DDCos(incipos, dir1, len)
-c            sample angle angle around dir1 (x,y axes arbitrary)
-      call rndc(u)
-      dir2.r(3) =  -(  (IMAG_P(CosZenith)- real(CosZenith) )*u +
+         call c3DV2DDCos(incipos, dir1, len)
+c            sample angle around dir1 (x,y axes arbitrary)
+         call rndc(u)
+         dir2.r(3) =  -(  (IMAG_P(CosZenith)- real(CosZenith) )*u +
      *             real(CosZenith) )    ! going down is negative
-      call rndc(u)
-      call kcossn(cs, sn)
-      sint = sqrt(1.d0-dir2.r(3)**2)
-      dir2.r(1) = - sint*cs      ! - is needed for going down ptcl.
-      dir2.r(2) = - sint*sn
+         call rndc(u)
+         call kcossn(cs, sn)
+         sint = sqrt(1.d0-dir2.r(3)**2)
+         dir2.r(1) = - sint*cs  ! - is needed for going down ptcl.
+         dir2.r(2) = - sint*sn
 c         convert it to xyz system
-      call ctransVectZ(dir1, dir2, incident.vec.w)
+         call ctransVectZ(dir1, dir2, incident.vec.w)
+c     @@@@@@@@@@@@@@@
+         call cscalerProd(incident.vec.w, dir1, cosx)
+         call rndc(ux)
+      enddo   
+c     @@@@@@@@@@@@@@@
+c                uv 5.51
+      if(Reverse .ne. 0) then
+c          we must revert the angle
+         dir2.r(1) = -dir2.r(1)
+         dir2.r(2) = -dir2.r(2)
+         dir2.r(3) = -dir2.r(3)
+         call ctransVectZ(dir1, dir2, incident.vec.w)
+      endif
+
 c         reset others  
       call cresetPrim2(incident)
       end
@@ -169,7 +193,6 @@ c
       record /track/ incident  ! input. you must give everything
 c                                       about primary
       
-
       call cmkInc2(incident)
       call ciniTracking
       call cinitStack
@@ -220,10 +243,15 @@ c          HeightOfInj (=100 km); Normally OffsetHeight is 0.
 c         if the primary is to be directed to different height
 c         than the detector, make it non zero.
       zenithcos = - AngleAtObsCopy.r(3)
+      if(ObsPlane .ne. spherical) then
+         to100km = clenbetween2h(
+     *    ObsSites(NoOfSites).pos.radiallen + OffsetHeight, 
+     *    Eradius + HeightOfInj,  zenithcos )  ! we need zenith angle here
+      else
+c           dummy 
+         to100km = 10000.
+      endif 
 
-      to100km = clenbetween2h(
-     *  ObsSites(NoOfSites).pos.radiallen + OffsetHeight, 
-     *  Eradius + HeightOfInj,  zenithcos )  ! we need zenith angle here
 
 c         primary is going upward even if Reverse = 0.
       Upgoing = Reverse .eq. 0 .and. zenithcos .lt. 0. 
@@ -232,12 +260,14 @@ c         primary is going upward even if Reverse = 0.
       if(( Reverse .eq. 0 .and. zenithcos .lt. 0. 
      *  .and.  HeightOfInj .gt. ObsSites(NoOfSites).pos.height)
      *  .or. ( Reverse .ne. 0 .and. zenithcos .gt. 0.)) then
+         if(Obsplane .ne. spherical) then
 c            distance to the conjugate point
-         to100km = to100km -
-     *    2*(ObsSites(NoOfSites).pos.radiallen + OffsetHeight)*
-     *    zenithcos
+            to100km = to100km -
+     *       2*(ObsSites(NoOfSites).pos.radiallen + OffsetHeight)*
+     *       zenithcos
 c             we should go reversed direction
-         to100km = - to100km
+            to100km = - to100km
+         endif
       endif
 c           injection point
       do i = 1, 3
@@ -256,10 +286,25 @@ c          set time 0
 
 c           Next obs site
       if( Upgoing ) then
-         incident.where = NoOfSites
+ccc         incident.where = NoOfSites
+         do i = NoOfSites, 1, -1
+            if( HeightOfInj .lt. ObsSites(i).pos.height ) then
+               incident.where = i
+               goto 33
+            endif
+         enddo
+         incident.where = 0
       else
-         incident.where = 1     
+ccc         incident.where = 1     
+         do i = 1, NoOfSites
+            if( HeightOfInj .gt. ObsSites(i).pos.height ) then
+               incident.where = i
+               goto 33
+            endif
+         enddo
+         incident.where = NoOfSites + 1
       endif
+ 33   continue
 
       incident.asflag = 0
       incident.pos.colheight = Infty  ! latest nuc. collision height
@@ -270,7 +315,6 @@ c           if OffsetHight=0
       if(OffsetHeight .eq. 0. .and. ObsPlane .ne. spherical) then
          if(zenithcos .ge. 0. .or. Upgoing) then
             do i = 1, NoOfSites-1
-
                len = clenbetween2h(
      *              ObsSites(NoOfSites).pos.radiallen, 
      *              ObsSites(i).pos.radiallen,
@@ -284,7 +328,6 @@ c           if OffsetHight=0
                enddo
             enddo
             do i = 1, NoOfASSites-1
-
                len = clenbetween2h(
      *         ASObsSites(NoOfASSites).pos.radiallen, 
      *         ASObsSites(i).pos.radiallen,
@@ -370,15 +413,24 @@ c#include  "Zearth.h"
       
       record /track/from   ! input.  track to be origin
  
-      real*8 clenbetween2h
-      integer i
+      real*8 leng
+c        clenbetween2h, leng
+      integer i, icon
 
       do i = 1, NoOfSites
-
-         ObsSites(i).minitime =
-     *       clenbetween2h(from.pos.radiallen, 
-     *       ObsSites(i).pos.radiallen,
-     *       from.vec.coszenith)    ! actually this is in m.
+c         ObsSites(i).minitime =
+c     *       clenbetween2h(from.pos.radiallen, 
+c     *       ObsSites(i).pos.radiallen,
+c     *       from.vec.coszenith)    ! actually this is in m.
+         call clenbetw2h(from.pos.radiallen, 
+     *       ObsSites(i).pos.radiallen, from.vec.coszenith,
+     *       leng, icon)     ! actually leng is in m.
+         if(icon .eq. 0) then
+            ObsSites(i).minitime = leng
+         else
+c              icon !=0 ==> light cannot come with this angle
+            ObsSites(i).minitime = 1.d10
+         endif
       enddo
       from.t = 0.    ! reset time of incident track.
       end
@@ -451,3 +503,54 @@ c
 
       pos.sys = 'xyz'
       end
+c       ************************* see if geomagnetic cut or not.
+        subroutine cifCutOff(icon)
+        implicit none
+#include "Zobs.h"
+#include "Zobsp.h"
+#include "Ztrack.h"
+#include "ZrigCut.h"
+
+         integer icon   !   output. 0 ==> not cut. 1 ==> cut.
+
+         record /coord/ angleatOb
+         record /track/ inc
+
+         real*8 p, rig, zen, azm, theta,  prob, u
+
+         call cqIncident(inc, angleatOb)
+
+         if(inc.p.charge .eq. 0) then
+            icon = 0
+         else
+c            angleatOb is down going ptcl's one, change sign
+            angleatOb.r(1) = - angleatOb.r(1)
+            angleatOb.r(2) = - angleatOb.r(2)
+            angleatOb.r(3) = - angleatOb.r(3)
+c             convert to theta fai in deg
+            call kdir2deg(angleatOb.r(1), angleatOb.r(2),
+     *      angleatOb.r(3), theta, azm)
+c
+c               azm is given from the current x-axis  (+ is counter
+c              clock wise) The x-axis is XaxisFromSouth
+c            degrees from the south in counter clockwise.
+c            convert azm so that measured from the south
+c
+            azm = mod(azm+ XaxisFromSouth, 360.d0)
+            if(ZenValue .eq. 'cos') then
+c                table zenith is in cos
+               zen = angleatOb.r(3)
+            else
+               zen = theta
+            endif
+            p = sqrt(inc.p.fm.p(4)**2 - inc.p.mass**2)
+            rig = p/abs(inc.p.charge)
+            call crigCut(azm, zen, rig, prob)
+            call rndc(u)
+            if(u .lt. prob) then
+               icon = 0
+            else
+               icon = 1
+            endif
+         endif
+         end
